package com.blazinggames.ld25
{
	import com.blazinggames.gamejam.data.Itterator;
	import com.blazinggames.gamejam.data.TileMap;
	import com.blazinggames.gamejam.layers.starling.IScreen;
	import com.blazinggames.gamejam.layers.starling.ScreenManager;
	import com.blazinggames.gamejam.layers.starling.TileMapRenderer;
	import com.blazinggames.gamejam.layers.starling.WorldSprite;
	import com.blazinggames.ld25.res.ResMan;
	
	import flash.geom.Rectangle;
	import flash.ui.Keyboard;
	import flash.utils.Timer;
	import flash.utils.getTimer;
	
	import starling.display.Sprite;
	import starling.text.TextField;
	import starling.textures.TextureAtlas;
	
	public class CorralScreen extends Sprite implements IScreen
	{
		public static const OBSTACLE_RATE:Number = 0.03;
		private static const GNOME_TILES_PER_INTERVAL:Number = 0.1;
		private static const SCORE_TEXT:String = "Score: ";
		private static const TIME_TEXT:String = "Time:";
		
		private var _screenManager:ScreenManager;
		private var _tileMap:TileMap;
		private var _mapRenderer:TileMapRenderer;
		private var _mapViewport:Rectangle;
		private var _gnome:Gnome;
		private var _reindeer:Array;
		
		private var _score:int;
		private var _highScore:int;
		private var _levelStartTime:int;
		private var _intervalsSinceLastHUD:int;
		
		private var _scoreText:TextField;
		private var _deerCountText:TextField;
		private var _timeText:TextField;

		private var _gameInProgress:Boolean;
		
		public function CorralScreen(screenManager:ScreenManager, params:Object)
		{
			super();
			var ta:TextureAtlas = ResMan.getTileset();
			_tileMap = new TileMap(64,64);
			_mapRenderer = new TileMapRenderer(_tileMap, ta.getTextures("tileset"));
			_mapRenderer.setViewportSize(640,480);
			_mapViewport = new Rectangle(0,0,20,15);
			_mapRenderer.renderLayer(_mapViewport);
			addChild(_mapRenderer);

			reset(screenManager, params);
			_gnome = new Gnome();
			addChild(_gnome);
			_reindeer = new Array();
			generateCorral(2);

			_score = 0;
			_scoreText = new TextField(200,32, SCORE_TEXT, "_sans", 24, 0x777700, true);
			addChild(_scoreText);
			_deerCountText = new TextField(200,32, "?", "_sans", 24, 0x777700, true);
			_deerCountText.x = 210;
			addChild(_deerCountText);
			_timeText = new TextField(200,32, TIME_TEXT, "_sans", 24, 0x777700, true);
			_timeText.x = 420;
			addChild(_timeText);
		}
		
		public function generateCorral(spawnCount:int):void
		{
			var cntrX:int, cntrY:int, tileID:int;
		
			// build the fence
			for (cntrX = 1; cntrX < 32; ++cntrX) {
				_tileMap.setTile(cntrX * 2 - 1, 0, 10);
				_tileMap.setTile(cntrX * 2, 0, 11);
				_tileMap.setTile(cntrX * 2 - 1, 63, 10);
				_tileMap.setTile(cntrX * 2, 63, 11);

				_tileMap.setTile(0, cntrX * 2 - 1, 12);
				_tileMap.setTile(0, cntrX * 2, 13);
				_tileMap.setTile(63, cntrX * 2 - 1, 12);
				_tileMap.setTile(63, cntrX * 2, 13);
			}
			_tileMap.setTile(0,0,14);
			_tileMap.setTile(0,63,16);
			_tileMap.setTile(63,0,15);
			_tileMap.setTile(63,63,17);
			_tileMap.setTile(31,0,8);
			_tileMap.setTile(32,0,9);
			_tileMap.setTile(31,0,8);
			_tileMap.setTile(32,0,9);
			
			// fill the land
			for (cntrX = 2; cntrX < 62; ++cntrX) {
				for (cntrY = 2; cntrY < 62; ++cntrY) {
					if (Math.random() < OBSTACLE_RATE)
						tileID = Math.floor(Math.random() * 2) + 18;
					else
						tileID = Math.floor(Math.random() * 8);
					_tileMap.setTile(cntrX, cntrY, tileID);
				}
			}
			_gnome.worldX = 32;
			_gnome.worldY = 1.5;
			
			while (_reindeer.length > 0)
				removeChild(_reindeer.pop());
			var deer:Deer;
			for (var cntr:int = 0; cntr < spawnCount; ++cntr) {
				deer = new Deer();
				do {
					deer.worldX = Math.random() * 62 + 1;
					deer.worldY = Math.random() * 61 + 2;
				} while (canEnterTile(deer.worldX, deer.worldY, deer) == false);
				addChild(deer);
				_reindeer[cntr] = deer;
			}

			_levelStartTime = getTimer();
			_gameInProgress = true;
			_score = 0;
		}
		
		public function reset(screenManager:ScreenManager, params:Object):void
		{
			_screenManager = screenManager;
		}
		
		public function handleTouch(touchX:int, touchY:int, touchtype:int):void
		{
			if ((_gameInProgress == false) && (touchtype == ScreenManager.TOUCHTYPE_ENDED)) {
				_screenManager.changeGameState(ScreenManager.GAMESTATE_GAME_OVER);
			}
		}
		
		public function handleKeyboard(keycode:int, isDown:Boolean, flags:uint):void
		{
			switch (keycode) {
				case Keyboard.UP:
				case Keyboard.W:
					_gnome.setDirection(Gnome.NORTH, isDown);
					break;
				case Keyboard.DOWN:
				case Keyboard.S:
					_gnome.setDirection(Gnome.SOUTH, isDown);
					break;
				case Keyboard.LEFT:
				case Keyboard.A:
					_gnome.setDirection(Gnome.WEST, isDown);
					break;
				case Keyboard.RIGHT:
				case Keyboard.D:
					_gnome.setDirection(Gnome.EAST, isDown);
					break;
				case Keyboard.ENTER:
					if ((_gameInProgress == false) && ( ! isDown))
						_screenManager.changeGameState(ScreenManager.GAMESTATE_GAME_OVER);
					break;
			}
		}

		public function canEnterTile(targetX:Number, targetY:Number, sprite:WorldSprite):Boolean
		{
			var canEnter:Boolean = false;
			var radius:Number = sprite.radius;
//			if (_tileMap.getTile(targetX, targetY) < 10) {
			if ( (_tileMap.getTile(targetX-radius, targetY-radius) < 10) && (_tileMap.getTile(targetX+radius, targetY-radius) < 10) &&
						(_tileMap.getTile(targetX-radius, targetY+radius) < 10) && (_tileMap.getTile(targetX+radius, targetY+radius) < 10)) {
				canEnter = true;
				var deer:Deer;
				var itterator:Itterator = new Itterator(_reindeer);
				while (deer = itterator.next() as Deer) {
					if ((deer != sprite) && (WorldSprite.findDistanceSquared(deer.worldX, deer.worldY, targetX, targetY) < 2))
						canEnter = false;
				}
			}

			return canEnter;
		}

		public function isExit(targetX:Number, targetY:Number):Boolean
		{
			var tileID:int = _tileMap.getTile(targetX, targetY);
			if ((tileID == 8) || (tileID == 9))
				return true;
			return false;
		}
		
		public function update(params:Object=null):void
		{
			if ( !_gameInProgress)
				return;
			var deer:Deer;
			var targetX:Number, targetY:Number;
			var itterator:Itterator = new Itterator(_reindeer);
			while ((deer = itterator.next() as Deer) != null) {
				if (deer.handleMovement(_gnome, this) == false) {
					itterator.remove();
					removeChild(deer);
					_score += 10;
					ResMan.playSound("ExitSound");
				}
			}

			targetX = _gnome.worldX;
			targetY = _gnome.worldY;
			if (_gnome.isMoving(Gnome.NORTH)) targetY -= GNOME_TILES_PER_INTERVAL;
			if (_gnome.isMoving(Gnome.EAST)) targetX += GNOME_TILES_PER_INTERVAL;
			if (_gnome.isMoving(Gnome.SOUTH)) targetY += GNOME_TILES_PER_INTERVAL;
			if (_gnome.isMoving(Gnome.WEST)) targetX -= GNOME_TILES_PER_INTERVAL;
			if (canEnterTile(targetX, targetY, _gnome)) {
				_gnome.worldX = targetX;
				_gnome.worldY = targetY;
			}
			_mapViewport.x = _gnome.worldX - 10;
			if (_mapViewport.x < 0) _mapViewport.x = 0;
			if ((_mapViewport.x + _mapViewport.width) > 64) _mapViewport.x = 44;
			_mapViewport.y = _gnome.worldY - 7.5;
			if (_mapViewport.y < 0) _mapViewport.y = 0;
			if ((_mapViewport.y + _mapViewport.height) > 64) _mapViewport.y = 49;
			
			_mapRenderer.renderLayer(_mapViewport);
			_gnome.x = (_gnome.worldX - _mapViewport.x) * 32;
			_gnome.y = (_gnome.worldY - _mapViewport.y) * 32;
			_gnome.update();
			for (var cntr:int = 0; cntr < _reindeer.length; ++cntr) {
				deer = _reindeer[cntr] as Deer;
				deer.x = (deer.worldX - _mapViewport.x) * 32;
				deer.y = (deer.worldY - _mapViewport.y) * 32;				
			}
			
			++_intervalsSinceLastHUD;
			if (_intervalsSinceLastHUD > 100) {
				_intervalsSinceLastHUD = 0;
				var remaining:int = 100 - (getTimer() - _levelStartTime) / 1000;
				_timeText.text = TIME_TEXT + remaining;
				_scoreText.text = SCORE_TEXT + _score;
				_deerCountText.text = "" + _reindeer.length;
				if (remaining < 1) {
					ResMan.playSound("TimeUpSound");
					var message:String = "Time has ran out!\n";
					if (_score > _highScore) {
						message = message + "New high score!!!";
						_highScore = _score;
					} else {
						message = message = "high score: " + _highScore;
					}
					_screenManager.showOverlay(message);
					_gameInProgress = false;
				}
			}
		}
		
	}
}